# Copyright: 2007-2009 Paul Obermeier (obermeier@poSoft.de)
# Distributed under BSD license.

package provide tcomexcel 0.2

namespace eval ::Excel {

    proc CopyChartToClipboard { chartId } {
        $chartId CopyPicture $::Excel::xlScreen $::Excel::xlBitmap \
                             $::Excel::xlScreen
    }

    proc SaveChartToFile { chartId fileName { filterType "GIF" } } {
        set fileName [file nativename $fileName]
        $chartId Export $fileName $filterType
    }

    proc SetChartSize { worksheetId chartId width height } {
        # The name of the chart is returned as a long name containing of
        # the worksheet name and the diagram name. As both the worksheet
        # name and the diagram name may have spaces, it is difficult to 
        # distinguish bewtween the two.
        # We assume that the last two words are the diagram name, which is
        # generated by Excel as "Diagram #" by default.
        # Using the long name as index to the Item property does not work.
        set chartFullName [$chartId Name]
        set chartSplit [split $chartFullName]
        append chartName [lindex $chartSplit end-1] " "
        append chartName [lindex $chartSplit end]

        set shapes [$worksheetId Shapes]
        set chart [$shapes Item $chartName]
        # This is also an Excel mystery. After setting the width and height
        # to the correct size (i.e. use width and height unchanged), Excel
        # says, it has changed the shape to the correct size.
        # But the diagram as displayed and also the exported bitmap has a 
        # size 4/3 greater than expected. 
        # We correct for that discrepancy here by multiplying with 3/4.
        $chart Width  [expr $width  * 0.75]
        $chart Height [expr $height * 0.75]
    }

    # Set the minimum and maximum scale of both axes of a chart.
    proc SetChartScale { chartId xmin xmax ymin ymax } {
        set xaxis [[$chartId Axes] Item $::Excel::xlPrimary]
        $xaxis MinimumScale [expr $xmin]
        $xaxis MaximumScale [expr $xmax]
        set yaxis [[$chartId Axes] Item $::Excel::xlSecondary]
        $yaxis MinimumScale [expr $ymin]
        $yaxis MaximumScale [expr $ymax]
    }

    # Set the minimum scale of an axis of a chart.
    # The axisName parameter must be either "x" or "y".
    proc SetChartMinScale { chartId axisName value } {
        if { $axisName eq "x" || $axisName eq "X" } {
            set axis [[$chartId Axes] Item $::Excel::xlPrimary]
        } else {
            set axis [[$chartId Axes] Item $::Excel::xlSecondary]
        }
        $axis MinimumScale [expr $value]
    }

    # Set the maximum scale of an axis of a chart.
    # The axisName parameter must be either "x" or "y".
    proc SetChartMaxScale { chartId axisName value } {
        if { $axisName eq "x" || $axisName eq "X" } {
            set axis [[$chartId Axes] Item $::Excel::xlPrimary]
        } else {
            set axis [[$chartId Axes] Item $::Excel::xlSecondary]
        }
        $axis MaximumScale [expr $value]
    }

    # Place a chart as an object into a worksheet.
    proc PlaceChart { chartId worksheetId } {
        set newChartId [$chartId Location $::Excel::xlLocationAsObject \
                        [::Excel::GetWorksheetName $worksheetId]]
        return $newChartId
    }

    proc CreateChart { cellsId chartType } {
        set appId [GetApplicationId $cellsId]

        switch [::Excel::GetVersion $appId] {
            "12.0" {
                set worksheetId [$cellsId Worksheet]
                set chartId [[[$worksheetId Shapes] AddChart $chartType] Chart]
            }
            default {
                set chartId [[$appId Charts] Add]
                $chartId ChartType $chartType
            }
        }
        return $chartId
    }

    proc AddLineChart { cellsId numRows numCols { title "" } { markerSize 5 } } {
        # Create the line chart.
        set chartId [CreateChart $cellsId $::Excel::xlLineMarkers]

        set rangeId [SelectRangeByIndex $cellsId 2 2 \
                     [expr $numRows+1] [expr $numCols+1]]
        $chartId SetSourceData $rangeId $::Excel::xlColumns

        set xrangeId [SelectRangeByIndex $cellsId 2 1 [expr $numRows+1] 1]
        for { set i 1 } { $i <= $numCols } { incr i } {
            [[$chartId SeriesCollection] Item $i] XValues $xrangeId
            set name [GetCellValue $cellsId 1 [expr {$i +1}]]
            [[$chartId SeriesCollection] Item $i] Name $name
            [[$chartId SeriesCollection] Item $i] MarkerSize $markerSize
        }

        set axis [[$chartId Axes] Item $::Excel::xlPrimary]
        $axis HasTitle True
        [[$axis AxisTitle] Characters] Text [GetCellValue $cellsId 1 1]

        set axis [[$chartId Axes] Item $::Excel::xlSecondary]
        $axis HasTitle True
        [[$axis AxisTitle] Characters] Text "Values"

        if { $title ne "" } {
            $chartId HasTitle True
            [[$chartId ChartTitle] Characters] Text $title
        } else {
            $chartId HasTitle False
        }

        set plotArea [$chartId PlotArea]
        [$plotArea Interior] ColorIndex $::Excel::xlColorIndexNone


        return $chartId
    }

    proc AddPointChart { cellsId numRows col1 col2 { title "" } { markerSize 5 } } {
        # Create the point chart.
        set chartId [CreateChart $cellsId $::Excel::xlXYScatter]

        # Select the range of cells to be used as data.
        # Data of col1 is the X axis. Data of col2 is the Y axis.
        set rangeId [SelectRangeByIndex $cellsId 2 $col2 [expr $numRows+1] $col2]
        $chartId SetSourceData $rangeId $::Excel::xlColumns

        set xrangeId [SelectRangeByIndex $cellsId 2 $col1 [expr $numRows+1] $col1]
        [[$chartId SeriesCollection] Item 1] XValues $xrangeId

        [[$chartId SeriesCollection] Item 1] MarkerSize $markerSize

        # Set chart specific properties.
        # Switch of legend display.
        $chartId HasLegend False

        # Set the chart title string.
        if { $title ne "" } {
            $chartId HasTitle True
            [[$chartId ChartTitle] Characters] Text $title
        } else {
            $chartId HasTitle False
        }

        # Do not fill the chart interior area. Better for printing.
        set plotArea [$chartId PlotArea]
        [$plotArea Interior] ColorIndex $::Excel::xlColorIndexNone

        # Set axis specific properties.
        # Set the X axis description to cell col1 in row 1.
        set axis [[$chartId Axes] Item $::Excel::xlPrimary]
        $axis HasTitle True
        [[$axis AxisTitle] Characters] Text [GetCellValue $cellsId 1 $col1]
        # Set the display of major and minor gridlines.
        $axis HasMajorGridlines True
        $axis HasMinorGridlines False

        # Set the Y axis description to cell col2 in row 1.
        set axis [[$chartId Axes] Item $::Excel::xlSecondary]
        $axis HasTitle True
        [[$axis AxisTitle] Characters] Text [GetCellValue $cellsId 1 $col2]
        # Set the display of major and minor gridlines.
        $axis HasMajorGridlines True
        $axis HasMinorGridlines False

        return $chartId
    }

    proc AddRadarChart { cellsId numRows numCols { title "" } } {
        # Create the radar chart. Default is filled.
        set chartId [CreateChart $cellsId $::Excel::xlRadarFilled]

        # Select the range of cells to be used as data.
        set rangeId [SelectRangeByIndex $cellsId 2 2 \
                     [expr $numRows+1] [expr $numCols+1]]
        $chartId SetSourceData $rangeId $::Excel::xlColumns

        set xrangeId [SelectRangeByIndex $cellsId 2 1 [expr $numRows+1] 1]
        for { set i 1 } { $i <= $numCols } { incr i } {
            [[$chartId SeriesCollection] Item $i] XValues $xrangeId
            set name [GetCellValue $cellsId 1 [expr $i +1]]
            [[$chartId SeriesCollection] Item $i] Name $name
        }

        # Set chart specific properties.
        # Switch on legend display.
        $chartId HasLegend True

        # Set the chart title string.
        if { $title ne "" } {
            $chartId HasTitle True
            [[$chartId ChartTitle] Characters] Text $title
        } else {
            $chartId HasTitle False
        }

        # Do not fill the chart interior area. Better for printing.
        set plotArea [$chartId PlotArea]
        [$plotArea Interior] ColorIndex $::Excel::xlColorIndexNone

        # Set axis specific properties.
        set axis [[$chartId Axes] Item $::Excel::xlPrimary]
        # Set the display of major and minor gridlines.
        $axis HasMajorGridlines False
        $axis HasMinorGridlines False

        set axis [[$chartId Axes] Item $::Excel::xlSecondary]
        # Set the display of major and minor gridlines.
        $axis HasMajorGridlines True
        $axis HasMinorGridlines False

        return $chartId
    }
}
